<?php
/**
 * @author Marijan Šuflaj <msufflaj32@gmail.com>
 * @link http://www.php4every1.com
 */

/*
    Plugin Name: Tiny Table Of Content - TinyTOC
    Plugin URI: http://php4every1.com/scripts/tiny-table-of-contents-wordpress-plugin/
    Description: Advanced plugin for dynamic creation of "Table of content" for you post or pages.
    Version: 1.6.3
    Author: Marijan Šuflaj
    Author URI: http://www.php4every1.com
*/

/*
    Copyright 2009  Marijan Šuflaj  (email : msufflaj32@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

//Used for preventing direct view.
define('IN_PLUGIN', true);

//Current version.
define('CURR_VER', '1.6.3');

//Load config class.
require_once 'classes/config.php';

if (get_option('tinytoc_settings_enabled') === false) {
    require_once 'installTinyTOC.php';
}

$options = array(
    'tinytoc_settings_info'
);

$config = tinyConfig::getInstance()->get($options);

if (version_compare(CURR_VER, $config->tinytoc_settings_info->version, 'gt')) {
	require_once 'pluginUpgrade.php';
}

//Call function that hooks menus to admin panel.
add_action('admin_menu', 'hookMenus');

//Hook init function.
add_action('init',      'tinyTocAddButtons');
add_action('wp_footer', 'echoStyleSheets');

/**
 * Function create stylesheets.
 *
 * @return
 */
function echoStyleSheets() {
	$options = array(
		'tinytoc_settings_header',
		'tinytoc_settings_backtotop',
		'tinytoc_settings_tocstyle',
	);
	
	$config = tinyConfig::getInstance()->get($options);
?>
<!-- TinyTOC Stylesheets -->
<style type="text/css">
/* Header styles */
<?php echo $config->tinytoc_settings_header->css ?>

/* Back To Top button styles */
<?php echo $config->tinytoc_settings_backtotop->css ?>

/* Table Of Content styles */
<?php echo $config->tinytoc_settings_tocstyle->css ?>
</style>
<?php
}


/**
 * Init function
 *
 * @return
 */
function tinyTocAddButtons() {
	//Add button and plugin to tinyMCE editor.
	add_filter("mce_external_plugins", "tinyTocAddPlugin");
	add_filter('mce_buttons', 'tinyTocRegisterButton');
}

/**
 * Register button function.
 *
 * @param array $buttons Buttons that need to be enabled
 * @return array Buttons that need to be enabled
 */
function tinyTocRegisterButton($buttons) {
	array_push($buttons, 'tinyPlugin');
	return $buttons;
}

/**
 * Register plugin function
 *
 * @param array $plugin_array Plugins that need to be registered
 * @return array Plugins that need to be registered
 */
function tinyTocAddPlugin($plugin_array) {
	//Start output buffering.
	ob_start();
?>
(function()  {
   // Create a new plugin class
   tinymce.create('tinymce.plugins.ExamplePlugin', {
   createControl: function(n, cm) {
       switch (n) {
           case 'tinyPlugin':
               var mlb = cm.createListBox('tinyPlugin', {
                    title : 'TOC Levels',
                    onselect : function(v) {
                        var content = tinyMCE.activeEditor.selection.getContent({format : 'text'});
                        tinyMCE.activeEditor.selection.setContent('[tinytoc level="' + v + '"]' + content + '[/tinytoc]');
                    }
               });

               // Add some values to the list box
<?php
//Create levels.
$config = tinyConfig::getInstance()->get('tinytoc_settings_general');
$num = $config->tinytoc_settings_general->maxLevelNum + 1;
for ($i = 1; $i < $num; $i++) :
?>
                mlb.add('Level <?php echo $i; ?>', '<?php echo $i; ?>');
<?php
endfor;
?>

		        // Return the new listbox instance
		        return mlb;
	        }

	        return null;
	    }

    });

    // Register plugin with a short name
    tinymce.PluginManager.add('tinyPlugin', tinymce.plugins.ExamplePlugin);
})();

<?php

    //Stop output buffering and save content to file.
    $content = ob_get_clean();
    $file = fopen(dirname(__FILE__) . DIRECTORY_SEPARATOR . 'js' . DIRECTORY_SEPARATOR . 'tinyPlugin.js', 'w');
    fwrite($file, $content);
    fclose($file);

    $plugin_array['tinyPlugin'] = plugins_url('tiny-table-of-contents-tinytoc/js/tinyPlugin.js');

    return $plugin_array;

}

/*
 * Function that hooks menus to admin panel.
 *
 * @return
 */
function hookMenus()
{
	//Menu.
    add_menu_page('TinyTOC Sumary', 'TinyTOC', 8, 'tiny-table-of-contents-tinytoc/home.php');

    //Submenus.
    add_submenu_page(
		'tiny-table-of-contents-tinytoc/home.php',
		'TinyTOC Summary',
		'Summary',
		8,
		'tiny-table-of-contents-tinytoc/home.php'
	);
    add_submenu_page(
		'tiny-table-of-contents-tinytoc/home.php',
		'TinyTOC Settings',
		'Settings',
		8,
		'tiny-table-of-contents-tinytoc/settings.php'
	);

    $options = array(
        'tinytoc_settings_enabled',
        'tinytoc_settings_header',
        'tinytoc_settings_general',
        'tinytoc_settings_parse',
        'tinytoc_settings_backtotop',
        'tinytoc_settings_tocstyle',
        'tinytoc_chapter_styling',
        'tinytoc_settings_info'
    );

    tinyConfig::getInstance()->get($options);
}

$config = tinyConfig::getInstance()->get('tinytoc_settings_general');
add_action(
    'the_content',
    'startContentParsing',
    $config->tinytoc_settings_general->priority
);

/**
 * Function that start parsing.
 *
 * @param string $data Post content
 * @return string Parsed post content
 */
function startContentParsing($data) {
	//Get config.
	$config = tinyConfig::getInstance()->get(array(
		'tinytoc_settings_enabled',
		'tinytoc_settings_header',
		'tinytoc_settings_general',
		'tinytoc_settings_tocstyle',
		'tinytoc_chapter_styling',
		'tinytoc_settings_backtotop'
	));

	if ($config->tinytoc_settings_enabled !== '1') return $data;

	//Get post.
	global $post;

	//Split it by <!--nextpage-->.
	$allContent = explode('<!--nextpage-->', $post->post_content);
	
	//Number of pages.
	$numPages = count($allContent);

	//Get current page of multipage post.
	$currentPage = ((int) get_query_var('page') === 0) ? 1 : (int) get_query_var('page');

	// Group 0: Level Number, Group 1: Leveltext
	// String for TinyToc-Attributes
	$searchStringTinyToc = '/\b\[tinytoc[[:space:]]+level="([0-9]+)"\]([\S\s]*?)\[\/tinytoc\]\b/i';
	// String for TinyToc-Attributes
	$searchStringRegHtml = '/<h([1-6]+)>([\S\s]*?)<\/h(\1)>/i'; 
	$searchString = "";
	$config->tinytoc_settings_general->useStandardHtmlTags ? $searchString = $searchStringRegHtml : $searchString = $searchStringTinyToc;

	//Check if we need to parse it.
	if (!checkIfWeNeedToParse()) return $config->tinytoc_settings_general->removeWhenNotUsed ? preg_replace($searchStringTinyToc, '$2', $data) : $data;
	$urlStuffTemp = array();
	$urlStuff     = array();
	$match        = false;

	// Reset Levels to "1" on each Level
	$currentLevels = array_fill(0, 9, 0);
	$oldLevel      = 1;
	
	//Loop all pages and create TOC and extend Chapters, if necessary
	$completeMatchesArray  = array();
	$onlyLevelNumbersArray = array();
	$onlyHeadingTextsArray = array();
	$onlyPageNumbersArray  = array();
	$enumerationValueArray = array();
	
	for ($page = 0; $page < $numPages; $page++) {
		// Search for headings
		if (!preg_match_all($searchString, $allContent[$page], $urlStuffTemp)) { continue; }
		$match = true;
		
		// So, we found at least one match on this page.
		// All Matches are stored in $urlStuffTemp, where:
		// $urlStuffTemp[0] = array of all complete matches in correct order
		// $urlStuffTemp[1] = array of all 1st groups, so all found levels in correct order (1 1 1 2 3 3 3 2 3 3 3 1 1)
		// $urlStuffTemp[2] = array of all 2nd groups, so all found heading texts in correct order
		$completeMatchesArray  = array_merge($completeMatchesArray,  $urlStuffTemp[0]);
		$onlyLevelNumbersArray = array_merge($onlyLevelNumbersArray, $urlStuffTemp[1]);
		$onlyHeadingTextsArray = array_merge($onlyHeadingTextsArray, $urlStuffTemp[2]);
		$onlyPageNumbersArray  = array_merge($onlyPageNumbersArray,  array_fill(0, count($urlStuffTemp[2]), ($page+1)));
	}
	
	$toc = "";
	if ($config->tinytoc_settings_tocstyle->tocEnumerationStyle==1) { $toc .= "<ul>"; }
	for ($currentHeadingIndex=0; $currentHeadingIndex < count($completeMatchesArray); $currentHeadingIndex++) {
		// Basically we need to do these things:
		$currentHeadingLevel = $onlyLevelNumbersArray[$currentHeadingIndex];

		// 1. Create indentation Chapter Numbers (1., 1.1., 1.2., ...)
		if ($currentHeadingLevel > $oldLevel) {
			for ($i=$currentHeadingLevel-1;$i<count($currentLevels);$i++) { $currentLevels[$i] = 0; }
		}
		$currentLevels[$currentHeadingLevel-1] = $currentLevels[$currentHeadingLevel-1]+1;
		// construct chapter number as is
		for ($i=0;$i<$currentHeadingLevel;$i++) {
			$enumerationValueArray[$currentHeadingIndex] = $enumerationValueArray[$currentHeadingIndex] .  getNumberRepresentationForLevel($i+1, $currentLevels[$i]) . getTrainlingCharForLevel($i+1, ($i==$currentHeadingLevel-1));
		}
		$oldLevel = $currentHeadingLevel;
		
		// 2. Add Heading to TOC
		if ($config->tinytoc_settings_tocstyle->maxTocLevel>=$currentHeadingLevel) {
			$firstClass = "";
			$lastClass = "";
			$currentPageClass = ($currentPage==$onlyPageNumbersArray[$currentHeadingIndex] ? " current_page" : "");
			
			if ($currentHeadingLevel > $onlyLevelNumbersArray[getPreviousVisibleHeadingIndex($config->tinytoc_settings_tocstyle->maxTocLevel, $currentHeadingIndex, $onlyLevelNumbersArray)]) { $firstClass = " first_in_block"; }
			if ($currentHeadingLevel > $onlyLevelNumbersArray[getNextVisibleHeadingIndex($config->tinytoc_settings_tocstyle->maxTocLevel, $currentHeadingIndex, $onlyLevelNumbersArray)]) { $lastClass  = " last_in_block";  }

			if ($config->tinytoc_settings_tocstyle->tocEnumerationStyle==2) {
				$toc .= '<div class="tinytoc_level_'. $onlyLevelNumbersArray[$currentHeadingIndex] . $firstClass . $lastClass . $currentPageClass .'">';
				$toc .= $enumerationValueArray[$currentHeadingIndex] .' ';
			}
			if ($config->tinytoc_settings_tocstyle->tocEnumerationStyle==1) {
				if ($firstClass) { $toc .= '</ul><ul>'; }
				$toc .= '<li class="tinytoc_level_'. $currentHeadingLevel . $firstClass . $lastClass . $currentPageClass .'">';
			}
			if ($config->tinytoc_settings_general->useGoTo==1) {
				$toc .= createGoToUrl($onlyPageNumbersArray[$currentHeadingIndex], $currentPage, $currentHeadingLevel, $onlyHeadingTextsArray[$currentHeadingIndex]);
			} else {
				$toc .= $onlyHeadingTextsArray[$currentHeadingIndex];
			}
			if ($config->tinytoc_settings_tocstyle->tocEnumerationStyle==0) { $toc .= "<br />"; }
			if ($config->tinytoc_settings_tocstyle->tocEnumerationStyle==1) {
				$toc .= '</li>';
				if ($lastClass) { $toc .= "</ul><ul>"; }
			}
			if ($config->tinytoc_settings_tocstyle->tocEnumerationStyle==2) { $toc .= '</div>'; }
		}
	}
	if ($config->tinytoc_settings_tocstyle->tocEnumerationStyle==1) { $toc .= "</ul>"; }
	
	$toc = $config->tinytoc_settings_header->after . $toc;
	$toc = $config->tinytoc_settings_header->title . $toc;
	$toc = $config->tinytoc_settings_header->before . $toc;
	
	// Now we loop though the headings od the current page...
	$headingFirstIndex = 0;
	for ($i=0;$i<count($onlyPageNumbersArray);$i++) {
		if ($onlyPageNumbersArray[$i] == $currentPage) {
			$headingFirstIndex = $i;
			break;
		}
	}
	$headingLastIndexPlusOne = count($onlyPageNumbersArray);
	for ($i=0;$i<count($onlyPageNumbersArray);$i++) {
		if ($onlyPageNumbersArray[$i] == $currentPage+1) {
			$headingLastIndexPlusOne = $i;
			break;
		}
	}
	
	for ($currentHeadingIndex=$headingFirstIndex; $currentHeadingIndex < $headingLastIndexPlusOne; $currentHeadingIndex++) {

		if ($config->tinytoc_chapter_styling->useChapterEnumeration==1) {
			$text    = $completeMatchesArray[$currentHeadingIndex];
			$newText = $enumerationValueArray[$currentHeadingIndex] .' '. $onlyHeadingTextsArray[$currentHeadingIndex];
			if ( ($config->tinytoc_settings_general->tocOnAllPages || $currentPage == 1) && $config->tinytoc_settings_general->useBackToTop) {
				$newText .= $config->tinytoc_settings_backtotop->html;
			}
			$text = str_replace( $onlyHeadingTextsArray[$currentHeadingIndex], $newText, $text );
			$data = str_replace_first($completeMatchesArray[$currentHeadingIndex], $text, $data);
		}
		
		$idString = ' id="' . str_replace( array(' ', '.'), '-', htmlentities( strip_tags($onlyHeadingTextsArray[$currentHeadingIndex]) ) ) . '-' . $onlyLevelNumbersArray[$currentHeadingIndex] . '"';
		if ($config->tinytoc_settings_general->useStandardHtmlTags==1) {
			// When using HTML-H*-Tags, add the ID to the tag...
			$data = str_replace_first(substr($completeMatchesArray[$currentHeadingIndex], 0, 4), substr($completeMatchesArray[$currentHeadingIndex], 0, 3) . $idString .">", $data);
		}
	}
	
	$toc .= "<p>&nbsp;</p>";
	if (!(!$config->tinytoc_settings_general->tocOnAllPages && $currentPage > 1 )) {
		if (strpos($data, '[tinytoc]')) { $data = str_replace('[tinytoc]', $toc, $data); } else { $data = $toc . $data; }
	}
	return $data;
}

/**
 * Creates GoTo urls if needed.
 *
 * @param int $page Current page in loop
 * @param int $currPage Current page in URL
 * @param string $level Current level
 * @param string $title Current title
 * @return string Formated string
 */
function createGoToUrl($page, $currPage, $level, $title) {
	global $post;
	//If enabled GoTo feature, this will create urls for them.
	$return = '<a href="';
	$return .= (($page !== $currPage) ?
           (('' == get_option('permalink_structure') ||
               in_array($post->post_status, array('draft', 'pending'))
           ) ?  (get_permalink() . '&page=' . $page) :
           (trailingslashit(get_permalink()) . user_trailingslashit($page, 'single_paged'))) : '')
       . '#' . str_replace(
            array(' ', '.'),
            '-',
            htmlentities(strip_tags($title))
        ) . '-' . $level . '">' . $title . '</a>';

	return $return;
}

/**
 * Function that we use to check if we need to parse this request.
 *
 * @return bool True if we need to parse it, otherwise false
 */
function checkIfWeNeedToParse() {
	global $post;
	$config = tinyConfig::getInstance()->get('tinytoc_settings_parse');
	if (is_home() && !$config->tinytoc_settings_parse->parseHomePage)               return false;
	if (is_archive() && !$config->tinytoc_settings_parse->parseAnyArchive)          return false;
	if (is_date() && !$config->tinytoc_settings_parse->parseDateArchive)            return false;
	if (is_category() && !$config->tinytoc_settings_parse->parseCategoryArchive)    return false;
	if (is_search() && !$config->tinytoc_settings_parse->parseSearch)               return false;
	if (is_feed() && !$config->tinytoc_settings_parse->parseFeed)                   return false;
	if ($post->post_type === 'post' && !$config->tinytoc_settings_parse->parsePost) return false;
	if ($post->post_type === 'page' && !$config->tinytoc_settings_parse->parsePage) return false;
	if ( in_array($post->ID, $config->tinytoc_settings_parse->postExclude) || in_array($post->ID, $config->tinytoc_settings_parse->pageExclude) ) return false;
	return true;
}

/**
 * Function used to calculate Roman numbers from Integer
 *
 * @param array $matches PHP preg_replace_callback matches
 * @return string Formated string
 */
function numberToRoman($num) {
	$n = intval($num);
	$result = '';
	$lookup = array('M' => 1000, 'CM' => 900, 'D' => 500, 'CD' => 400, 'C' => 100, 'XC' => 90, 'L' => 50, 'XL' => 40, 'X' => 10, 'IX' => 9, 'V' => 5, 'IV' => 4, 'I' => 1);
 	foreach ($lookup as $roman => $value) {
		$matches = intval($n / $value);
		$result .= str_repeat($roman, $matches);
		$n = $n % $value;
	}
	return $result;
}

function numberToAlphabetical($num) {
	$chars  = "abcdefghijklmnopqrstuvwxyz";
	return $chars[$num];
}

function str_replace_first($search, $replace, $subject) {
	$pos = strpos($subject, $search);
	if ($pos !== false) { $subject = substr_replace($subject, $replace, $pos, strlen($search)); }
	return $subject;
}

function getNumberRepresentationForLevel($indentationLevel, $currentNumber) {
	$config = tinyConfig::getInstance()->get('tinytoc_chapter_styling');
	if (!$config->tinytoc_chapter_styling->useChapterEnumeration)               return "";
	if ($config->tinytoc_chapter_styling->maxEnumLevel<$indentationLevel)       return "";
	if ($config->tinytoc_chapter_styling->levelEnumerate[$indentationLevel]==0) return "";
	if ($config->tinytoc_chapter_styling->levelEnumerate[$indentationLevel]==1) return $currentNumber;
	if ($config->tinytoc_chapter_styling->levelEnumerate[$indentationLevel]==2) return strtolower(numberToRoman($currentNumber));
	if ($config->tinytoc_chapter_styling->levelEnumerate[$indentationLevel]==3) return strtoupper(numberToRoman($currentNumber));
	if ($config->tinytoc_chapter_styling->levelEnumerate[$indentationLevel]==4) return strtolower(numberToAlphabetical($currentNumber));
	if ($config->tinytoc_chapter_styling->levelEnumerate[$indentationLevel]==5) return strtoupper(numberToAlphabetical($currentNumber));
	return "";
}

function getTrainlingCharForLevel($indentationLevel, $isLastIndentationLevelInString) {
	$config = tinyConfig::getInstance()->get('tinytoc_chapter_styling');
	if (!$config->tinytoc_chapter_styling->useChapterEnumeration)                  return "";
	if ($config->tinytoc_chapter_styling->maxEnumLevel<$indentationLevel)          return "";
	if ($config->tinytoc_chapter_styling->levelTrailingChar[$indentationLevel]==0) return "";
	if ($config->tinytoc_chapter_styling->levelTrailingChar[$indentationLevel]==1) return ".";
	if ($config->tinytoc_chapter_styling->levelTrailingChar[$indentationLevel]==2) return ($isLastIndentationLevelInString ? "" : ".");
	if ($config->tinytoc_chapter_styling->levelTrailingChar[$indentationLevel]==3) return ")";
	if ($config->tinytoc_chapter_styling->levelTrailingChar[$indentationLevel]==4) return ($isLastIndentationLevelInString ? "" : ")");
	return "";
}

function getPreviousVisibleHeadingIndex($maxTocLevel, $currentIndex, $levelArray) {
	for ($i=$currentIndex-1;$i>0;$i--) {
		if ($levelArray[$i]<=$maxTocLevel) return $i;
	}
	return 0;
}

function getNextVisibleHeadingIndex($maxTocLevel, $currentIndex, $levelArray) {
	for ($i=$currentIndex+1;$i<count($levelArray);$i++) {
		if ($levelArray[$i]<=$maxTocLevel) return $i;
	}
	return count($levelArray);
}